from setuptools import setup, find_packages

setup(
    name="wagtail-links",
    version="1.0.2",
    author="David Burke",
    author_email="david@thelabnyc.com",
    description="Wagtail links provides a consistent way to refer to links in a wagtail page",
    license="Apache License",
    keywords="django wagtail",
    url="https://gitlab.com/thelabnyc/wagtail-links",
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'Environment :: Web Environment',
        'Framework :: Django',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Intended Audience :: Developers',
        "License :: OSI Approved :: Apache Software License",
    ],
    install_requires=[
        'wagtail>=1.6.2',
    ]
)




from django.core.exceptions import ValidationError
from wagtail.tests.utils import WagtailPageTests
from wagtail.wagtailcore.models import Page
from unittest.mock import patch


from .models import Link


class WagtailLinksTests(WagtailPageTests):
    def test_get_link(self):
        link = Link.objects.create(link_external="https://example.com")
        self.assertEqual(link.url, "https://example.com")

        link = Link.objects.create(link_relative="/example.com")
        self.assertEqual(link.url, "/example.com")

        page = Page.objects.first()
        link = Link.objects.create(link_page=page)
        self.assertEqual(link.url, "")

        link = Link.objects.create(django_view_name="admin:index")
        self.assertEqual(link.url, "/admin/")

    def test_link_validation(self):
        link = Link(
            link_external="https://example.com",
            link_relative="/foo",
        )
        with self.assertRaises(ValidationError):
            link.clean()

        link = Link()
        with self.assertRaises(ValidationError):
            link.clean()

        link = Link(django_view_name="doesnotexist")
        with self.assertRaises(ValidationError):
            link.full_clean()

    @patch('wagtail_links.models.logger')
    def test_log_broken_links(self, mock_logger):
        link = Link.objects.create(django_view_name="doesnotexist")
        link.url
        mock_logger.warning.assert_called_with(
            "Unable to reverse django url for link id 1")

    def test_cast_link_to_string(self):
        link = Link.objects.create(link_external="https://example.com")
        self.assertEqual("{}".format(link), "https://example.com")

        link = Link.objects.create(link_relative="/example.com")
        self.assertEqual("{}".format(link), "/example.com")

        page = Page.objects.first()
        link = Link.objects.create(link_page=page)
        self.assertEqual("{}".format(link), "Root")

        link = Link.objects.create(django_view_name="admin:index")
        self.assertEqual("{}".format(link), "admin:index")



from django.core.exceptions import ValidationError
from django.core.validators import RegexValidator
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.urls import reverse, NoReverseMatch

from wagtail.wagtailadmin.edit_handlers import PageChooserPanel, FieldPanel
from wagtail.wagtailsnippets.models import register_snippet

import logging

logger = logging.getLogger(__name__)


def validate_django_reverse(view_name):
    try:
        reverse(view_name)
    except NoReverseMatch:
        raise ValidationError(_('Invalid django view name'))


@register_snippet
class Link(models.Model):
    """
    A generic link that points somewhere else using various methods
    """
    link_external = models.URLField("External link", blank=True)
    link_relative = models.CharField(
        "Relative link",
        max_length=200,
        blank=True,
        validators=[
            RegexValidator(
                regex=r'^(?!www\.|(?:http|ftp)s?://|[A-Za-z]:\\|//).*',
                message='Not valid relative url',
            ),
        ]
    )
    link_page = models.ForeignKey(
        'wagtailcore.Page',
        null=True,
        blank=True,
        related_name='+'
    )
    django_view_name = models.CharField(
        max_length=255,
        blank=True,
        help_text="Name of django view for reverse lookup",
        validators=[
            validate_django_reverse,
        ],
    )

    panels = [
        FieldPanel('link_external'),
        FieldPanel('link_relative'),
        PageChooserPanel('link_page'),
        FieldPanel('django_view_name'),
    ]

    def __str__(self):
        if self.link_external:
            return self.link_external
        elif self.link_relative:
            return self.link_relative
        elif self.link_page:
            return str(self.link_page)
        return self.django_view_name

    @property
    def url(self):
        """ Get url for use in template """
        if self.link_external:
            return self.link_external
        elif self.link_relative:
            return self.link_relative
        elif self.link_page:
            url = self.link_page.url
            if url is None:
                return ''
            return url
        else:
            try:
                return reverse(self.django_view_name)
            except NoReverseMatch:
                logger.warning(
                    'Unable to reverse django url for link id {}'.format(
                        self.id)
                )
                return ''

    def clean(self):
        # Don't allow multiple to be used
        number_used = bool(self.link_external) + bool(self.link_relative) \
            + bool(self.link_page) + bool(self.django_view_name)
        if number_used > 1:
            raise ValidationError(_('Use only one link type'))
        if number_used == 0:
            raise ValidationError(_('Must use exactly one link type'))
